@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2017 Google Inc. All Rights Reserved.
+# Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 # limitations under the License.
 """Tests for yapf.file_resources."""
 
+import contextlib
 import os
 import shutil
 import tempfile
@@ -26,6 +27,15 @@ from yapf.yapflib import py3compat
 from yapftests import utils
 
 
+@contextlib.contextmanager
+def _restore_working_dir():
+  curdir = os.getcwd()
+  try:
+    yield
+  finally:
+    os.chdir(curdir)
+
+
 class GetDefaultStyleForDirTest(unittest.TestCase):
 
   def setUp(self):
@@ -62,9 +72,11 @@ class GetCommandLineFilesTest(unittest.TestCase):
 
   def setUp(self):
     self.test_tmpdir = tempfile.mkdtemp()
+    self.old_dir = os.getcwd()
 
   def tearDown(self):
     shutil.rmtree(self.test_tmpdir)
+    os.chdir(self.old_dir)
 
   def _make_test_dir(self, name):
     fullpath = os.path.normpath(os.path.join(self.test_tmpdir, name))
@@ -136,6 +148,106 @@ class GetCommandLineFilesTest(unittest.TestCase):
             os.path.join(tdir2, 'testfile2.py'),
         ]))
 
+  def test_find_with_excluded_hidden_dirs(self):
+    tdir1 = self._make_test_dir('.test1')
+    tdir2 = self._make_test_dir('test_2')
+    tdir3 = self._make_test_dir('test.3')
+    files = [
+        os.path.join(tdir1, 'testfile1.py'),
+        os.path.join(tdir2, 'testfile2.py'),
+        os.path.join(tdir3, 'testfile3.py'),
+    ]
+    _touch_files(files)
+
+    actual = file_resources.GetCommandLineFiles(
+        [self.test_tmpdir], recursive=True, exclude=['*.test1*'])
+
+    self.assertEqual(
+        sorted(actual),
+        sorted([
+            os.path.join(tdir2, 'testfile2.py'),
+            os.path.join(tdir3, 'testfile3.py'),
+        ]))
+
+  def test_find_with_excluded_hidden_dirs_relative(self):
+    """Test find with excluded hidden dirs.
+
+    A regression test against a specific case where a hidden directory (one
+    beginning with a period) is being excluded, but it is also an immediate
+    child of the current directory which has been specified in a relative
+    manner.
+
+    At its core, the bug has to do with overzelous stripping of "./foo" so that
+    it removes too much from "./.foo" .
+    """
+    tdir1 = self._make_test_dir('.test1')
+    tdir2 = self._make_test_dir('test_2')
+    tdir3 = self._make_test_dir('test.3')
+    files = [
+        os.path.join(tdir1, 'testfile1.py'),
+        os.path.join(tdir2, 'testfile2.py'),
+        os.path.join(tdir3, 'testfile3.py'),
+    ]
+    _touch_files(files)
+
+    # We must temporarily change the current directory, so that we test against
+    # patterns like ./.test1/file instead of /tmp/foo/.test1/file
+    with _restore_working_dir():
+
+      os.chdir(self.test_tmpdir)
+      actual = file_resources.GetCommandLineFiles(
+          [os.path.relpath(self.test_tmpdir)],
+          recursive=True,
+          exclude=['*.test1*'])
+
+      self.assertEqual(
+          sorted(actual),
+          sorted([
+              os.path.join(
+                  os.path.relpath(self.test_tmpdir), os.path.basename(tdir2),
+                  'testfile2.py'),
+              os.path.join(
+                  os.path.relpath(self.test_tmpdir), os.path.basename(tdir3),
+                  'testfile3.py'),
+          ]))
+
+  def test_find_with_excluded_dirs(self):
+    tdir1 = self._make_test_dir('test1')
+    tdir2 = self._make_test_dir('test2/testinner/')
+    tdir3 = self._make_test_dir('test3/foo/bar/bas/xxx')
+    files = [
+        os.path.join(tdir1, 'testfile1.py'),
+        os.path.join(tdir2, 'testfile2.py'),
+        os.path.join(tdir3, 'testfile3.py'),
+    ]
+    _touch_files(files)
+
+    os.chdir(self.test_tmpdir)
+
+    found = sorted(
+        file_resources.GetCommandLineFiles(
+            ['test1', 'test2', 'test3'],
+            recursive=True,
+            exclude=[
+                'test1',
+                'test2/testinner/',
+            ]))
+
+    self.assertEqual(found, ['test3/foo/bar/bas/xxx/testfile3.py'])
+
+    found = sorted(
+        file_resources.GetCommandLineFiles(
+            ['.'], recursive=True, exclude=[
+                'test1',
+                'test3',
+            ]))
+
+    self.assertEqual(found, ['./test2/testinner/testfile2.py'])
+
+  def test_find_with_excluded_current_dir(self):
+    with self.assertRaises(errors.YapfError):
+      file_resources.GetCommandLineFiles([], False, exclude=['./z'])
+
 
 class IsPythonFileTest(unittest.TestCase):
 
@@ -180,6 +292,22 @@ class IsPythonFileTest(unittest.TestCase):
     self.assertFalse(file_resources.IsPythonFile(file1))
 
 
+class IsIgnoredTest(unittest.TestCase):
+
+  def test_root_path(self):
+    self.assertTrue(file_resources.IsIgnored('media', ['media']))
+    self.assertFalse(file_resources.IsIgnored('media', ['media/*']))
+
+  def test_sub_path(self):
+    self.assertTrue(file_resources.IsIgnored('media/a', ['*/a']))
+    self.assertTrue(file_resources.IsIgnored('media/b', ['media/*']))
+    self.assertTrue(file_resources.IsIgnored('media/b/c', ['*/*/c']))
+
+  def test_trailing_slash(self):
+    self.assertTrue(file_resources.IsIgnored('z', ['z']))
+    self.assertTrue(file_resources.IsIgnored('z', ['z/']))
+
+
 class BufferedByteStream(object):
 
   def __init__(self):
